#include <global.h>

//aktualny stan przycisków
uint16_t btns_status = 0;

//ustawia bit przy danym przycisku po każdym kliknięciu
//a po wykonaniu akcji zeruje go
uint16_t btns_acts = 0;

//zmienna odliczająca, wykorzystywana przy przytrzymywaniu przycisku
uint8_t __msElapsed = 0;

//zmienna odliczająca, wykorzystywana przy wykluczaniu iskrzenia
uint8_t __delayElapsed = 0;


/**
 * Sprawdza stan wciśnięcia przycisków i ustawia odpowiednie bity
 * w zmiennej status
 */
void checkButton(uint8_t *port, uint8_t pin, uint8_t id) {
if(bit_is_clear(btns_status,id) && bit_is_clear(*port, pin))  {
  //przycisk wciśnięty
  btns_status |= _BV(id);
  if(__delayElapsed==0)
	btns_acts |= _BV(id);
  __msElapsed = BTN_REPEAT_WAIT;
} else if(bit_is_set(*port, pin)) {
  //przycisk został zwolniony
  #ifdef BTN_DELAY
  if(bit_is_set(btns_status, id) && __delayElapsed==0) {
    __delayElapsed = BTN_DELAY;
  }
  #endif
  btns_status &= ~_BV(id);//odznacz w tablicy wciśniętych
}
}

/**
 * Po odliczeniu wartości __msElapsed przepisuje wartość status do acts,
 * dzięki temu uzyskany jest efekt "kliknięcia" przy przytrzymaniu przycisku/ów
 */ 
void btns_timeElapsed() {
  if(__msElapsed>0) __msElapsed --;
  if(__delayElapsed>0) __delayElapsed--;

  //przycisk wciśnięty (trzymany)
  if(__msElapsed==0) {
    btns_acts = btns_status;
    __msElapsed = BTN_REPEAT_BREAK;
  }
}

/**
 * Sprawdza, które przyciski zostały kliknięte, a nie wykonana została jeszcze
 * ich akcja oraz wywołuje funkcję przekazaną jako parametr, z parametrem BTN_ID
 */
void checkClicked(void (*f)(uint8_t)) {
  uint8_t i;
    
  for(i=0; i<=BTNS_LAST; i++) {
    if(bit_is_set(btns_acts, i) ) {
      btns_acts &= ~_BV(i);
      #ifdef BTNS_SAFE_ONCLICK 
      cli();
      #endif
      (*f)(i);
      #ifdef BTNS_SAFE_ONCLICK 
      sei();
      #endif
    }
  }
}

/**
 * Sprawdza kolejno wszystkie przyciski
 */
void checkButtons() {
  #ifdef BTN_0_PORT
  checkButton((uint8_t *)(&PIN(BTN_0_PORT)), BTN_0_PIN, 0);
  #ifdef BTN_1_PORT
  checkButton((uint8_t *)(&PIN(BTN_1_PORT)), BTN_1_PIN, 1);
  #ifdef BTN_2_PORT
  checkButton((uint8_t *)(&PIN(BTN_2_PORT)), BTN_2_PIN, 2);
  #ifdef BTN_3_PORT
  checkButton((uint8_t *)(&PIN(BTN_3_PORT)), BTN_3_PIN, 3);
  #ifdef BTN_4_PORT
  checkButton((uint8_t *)(&PIN(BTN_4_PORT)), BTN_4_PIN, 4);
  #ifdef BTN_5_PORT
  checkButton((uint8_t *)(&PIN(BTN_5_PORT)), BTN_5_PIN, 5);
  #ifdef BTN_6_PORT
  checkButton((uint8_t *)(&PIN(BTN_6_PORT)), BTN_6_PIN, 6);
  #ifdef BTN_7_PORT
  checkButton((uint8_t *)(&PIN(BTN_7_PORT)), BTN_7_PIN, 7);
  #endif
  #endif
  #endif
  #endif
  #endif
  #endif
  #endif
  #endif
}

void btns_init() {
  #ifdef BTN_0_PORT
  DDR(BTN_0_PORT) &= ~_BV(BTN_0_PIN);
  BTN_0_PORT |= _BV(BTN_0_PIN);
  #ifdef BTN_1_PORT
  DDR(BTN_1_PORT) &= ~_BV(BTN_1_PIN);
  BTN_1_PORT |= _BV(BTN_1_PIN);
  #ifdef BTN_2_PORT
  DDR(BTN_2_PORT) &= ~_BV(BTN_2_PIN);
  BTN_2_PORT |= _BV(BTN_2_PIN);
  #ifdef BTN_3_PORT
  DDR(BTN_3_PORT) &= ~_BV(BTN_3_PIN);
  BTN_3_PORT |= _BV(BTN_3_PIN);
  #ifdef BTN_4_PORT
  DDR(BTN_4_PORT) &= ~_BV(BTN_4_PIN);
  BTN_4_PORT |= _BV(BTN_4_PIN);
  #ifdef BTN_5_PORT
  DDR(BTN_5_PORT) &= ~_BV(BTN_5_PIN);
  BTN_5_PORT |= _BV(BTN_5_PIN);
  #ifdef BTN_6_PORT
  DDR(BTN_6_PORT) &= ~_BV(BTN_6_PIN);
  BTN_6_PORT |= _BV(BTN_6_PIN);
  #ifdef BTN_7_PORT
  DDR(BTN_7_PORT) &= ~_BV(BTN_7_PIN);
  BTN_7_PORT |= _BV(BTN_7_PIN);
  #endif
  #endif
  #endif
  #endif
  #endif
  #endif
  #endif
  #endif


  #ifdef BTN_USE_TIMER
  #if BTN_USE_TIMER == 2
  TCCR2 = _BV(CS21)|_BV(CS20);
  TCNT2 = 0x100 - 250;
  TIMSK |= _BV(TOIE2);
  #endif
  #if BTN_USE_TIMER == 1
  TCCR0 = _BV(CS01)|_BV(CS00);
  TCNT0 = 0x100 - 250;
  TIMSK |= _BV(TOIE0);
  #endif
  #endif
}


#ifdef BTN_USE_TIMER
volatile uint8_t btns_ms10 = 0;

#if BTN_USE_TIMER == 2
SIGNAL(SIG_OVERFLOW2) {
  TCNT2 = 0x100 - 250;
  if(++btns_ms10 > 10) {
    btns_timeElapsed();
    btns_ms10 = 0;
  }
}
#endif 

#if BTN_USE_TIMER == 1
SIGNAL(SIG_OVERFLOW0) {
  TCNT0 = 0x100 - 250;
  if(++btns_ms10 > 10) {
    btns_timeElapsed();
    btns_ms10 = 0;
  }
}
#endif 

#endif
